@@ -14,7 +14,8 @@ logger = logging.getLogger("aiokafkadaemon")
 
 class Worker:
 
-    _boto3_client = None
+    _sf_client = None
+    _sqs_client = None
 
     def __init__(
         self,
@@ -27,8 +28,15 @@ class Worker:
         on_run=None,
         sasl_opts={},
         consumer_opts={},
+        producer_opts={},
+        sqs_opts={},
     ):
-        loop = asyncio.get_event_loop()
+        try:
+            # only called in async func when the worker is created
+            loop = asyncio.get_running_loop()
+        except RuntimeError:
+            loop = asyncio.get_event_loop()
+
         self._kafka_broker_addr = kafka_broker_addr
         self._kafka_group_id = kafka_group_id
         self._consumer_topic = consumer_topic
@@ -37,6 +45,7 @@ class Worker:
             self._producer_topic = consumer_topic
         self._consumer = None
         self._producer = None
+        self._exit = False
 
         # Automatically reads environment variables
         #       AWS_ACCESS_KEY_ID
@@ -45,16 +54,22 @@ class Worker:
         #       AWS_DEFAULT_REGION
         # https://boto3.amazonaws.com/v1/documentation/api/latest/guide/configuration.html#environment-variables
         try:
-            self._boto3_client = boto3.client("stepfunctions")
+            self._sf_client = boto3.client("stepfunctions")
+
+            # only initiate here (low footprint), whether to use is implemented in on_run by subclass
+            self._sqs_client = boto3.client("sqs")
+            self._sqs_opts = sqs_opts
         except Exception as e:
-            logger.warning(f"Unable to connect to boto3 client for step functions: {e}")
+            logger.warning(f"Unable to connect to boto3 client: {e}")
 
         if create_consumer:
             self._consumer = Worker.make_consumer(
                 loop, kafka_broker_addr, kafka_group_id, sasl_opts, consumer_opts
             )
         if create_producer:
-            self._producer = Worker.make_producer(loop, kafka_broker_addr, sasl_opts)
+            self._producer = Worker.make_producer(
+                loop, kafka_broker_addr, sasl_opts, producer_opts
+            )
 
         # legacy code, to be removed in future releases
         if on_run:
@@ -97,7 +112,7 @@ class Worker:
         return consumer
 
     @classmethod
-    def make_producer(cls, loop, broker_addr, sasl_opts={}):
+    def make_producer(cls, loop, broker_addr, sasl_opts={}, producer_opts={}):
         """
         Creates an instance of the AIOKafka producer
         :param loop:
@@ -107,7 +122,10 @@ class Worker:
         logger.debug("Creating instance of producer")
         if not sasl_opts:
             producer = AIOKafkaProducer(
-                loop=loop, bootstrap_servers=broker_addr, compression_type="snappy"
+                loop=loop,
+                bootstrap_servers=broker_addr,
+                compression_type="snappy",
+                **producer_opts,
             )
         else:
             producer = AIOKafkaProducer(
@@ -119,6 +137,7 @@ class Worker:
                 sasl_plain_password=sasl_opts["password"],
                 security_protocol="SASL_SSL",
                 ssl_context=ssl.SSLContext(ssl.PROTOCOL_TLSv1_2),
+                **producer_opts,
             )
 
         logger.info("Producer connected to kafka on {}".format(broker_addr))
@@ -138,6 +157,9 @@ class Worker:
         """
         return await self.stop()
 
+    def should_exit(self):
+        return self._exit
+
     async def start(self):
         if self._consumer_topic:
             self._consumer.subscribe([self._consumer_topic])
@@ -159,10 +181,12 @@ class Worker:
 
     async def stop(self):
         logger.warning("System stopping, stopping consumer first")
+        self._exit = True
         await self.stop_kafka(False)
 
     async def clean_up_after_run(self):
         logger.warning("clean up after run")
+        self._exit = True
         await self.stop_kafka(True)
 
     async def run(self):
@@ -197,7 +221,7 @@ class Worker:
         then send a regular heartbeat when processing to let Step Functions
         know and not kill the process.
         """
-        if self._boto3_client is None or step_functions_task_id is None:
+        if self._sf_client is None or step_functions_task_id is None:
             return
         try:
             logger.info(
@@ -207,7 +231,7 @@ class Worker:
                 await asyncio.get_running_loop().run_in_executor(
                     pool,
                     partial(
-                        self._boto3_client.send_task_heartbeat,
+                        self._sf_client.send_task_heartbeat,
                         taskToken=step_functions_task_id,
                     ),
                 )
@@ -223,7 +247,7 @@ class Worker:
         then send back a relevant result (success or failure) with useful
         data for processing.
         """
-        if self._boto3_client is None or step_functions_task_id is None:
+        if self._sf_client is None or step_functions_task_id is None:
             return
         try:
             with concurrent.futures.ThreadPoolExecutor() as pool:
@@ -235,7 +259,7 @@ class Worker:
                     await asyncio.get_running_loop().run_in_executor(
                         pool,
                         partial(
-                            self._boto3_client.send_task_failure,
+                            self._sf_client.send_task_failure,
                             taskToken=step_functions_task_id,
                             error=str(result.get("code")),
                             cause=str(result.get("error")),
@@ -248,7 +272,7 @@ class Worker:
                     await asyncio.get_running_loop().run_in_executor(
                         pool,
                         partial(
-                            self._boto3_client.send_task_success,
+                            self._sf_client.send_task_success,
                             taskToken=step_functions_task_id,
                             output=result_data.decode("utf-8"),
                         ),
